/////////////////////////////////////////////////////////////////////////
//	"devtest_dev.cpp"
//------------------------------------------------------------------------
//	$Id: devtest_dev.cpp 57 2003-12-11 15:32:55Z  $
//////////////////////////////////////////////////////////////////////////

#include	<stdlib.h>
#include	<stdio.h>
#include	<string.h>
#if	defined(LINUX)
# include	"linux/hdreg.h"
#endif
#include	"devtest.h"
#include	"defs.h"

//////////////////////////////////////////////////////////////////////////
//	Print Device Information

int		DevTest::PrintDevInfo(DevTest::DevParam & devParam)
{
	unsigned					pathId, targetId, lun;
	char						t1[80],t2[80],t3[80];

	if ( devParam.inqData == NULL )
		return -1;
	pathId   = devParam.inqData->GetPathId();
	targetId = devParam.inqData->GetTargetId();
	lun      = devParam.inqData->GetLun();

#if	defined(WIN32)
	{
		IO_SCSI_CAPABILITIES	capa;

		if ( this->devCtl.ScsiGetCapabilities(capa) == 0 && capa.Length >= 24 )
		{
			//  ----+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8
			// "    Disk Capabilities                ##########"
			// "    <-----------------------------------------> ##########"
			PutFmt(4, "Disk Capabilities"            , "", "");
			PutFmt(8, "Maximum Transfer Length"      , Fmt(t2,"%u",(unsigned)capa.MaximumTransferLength      ), "");
			PutFmt(8, "Maximum Physical Pages"       , Fmt(t2,"%u",(unsigned)capa.MaximumPhysicalPages       ), "");
			PutFmt(8, "Supported Asynchronous Events", Fmt(t2,"%u",(unsigned)capa.SupportedAsynchronousEvents), "");
			PutFmt(8, "Alignment Mask"               , Fmt(t2,"%u",(unsigned)capa.AlignmentMask              ), "");
			PutFmt(8, "Tagged Queuing"               , Fmt(t2,"%u",(unsigned)capa.TaggedQueuing    ? 1 : 0   ), "");
			PutFmt(8, "Adapter Scans Down"           , Fmt(t2,"%u",(unsigned)capa.AdapterScansDown ? 1 : 0   ), "");
			PutFmt(8, "Adapter Uses PIO"             , Fmt(t2,"%u",(unsigned)capa.AdapterUsesPio   ? 1 : 0   ), "");
		}
	}
#endif
#if	defined(WIN32)
	{
		DISK_GEOMETRY	geo;

		if ( this->devCtl.DiskGetDriveGeometry(geo) == 0 )
		{
			PutPos(4        , "Disk Drive Geometry",
			       COLUMN2-4, Fmt(t1,"Media Type:%02XH", (unsigned)geo.MediaType),
			       COLUMN3  , this->devCtl.GetMediaTypeName(geo.MediaType),
			       0);
			sprintf(t1," %"FI64"u/%u/%u : %u"
			  ,    (INT64)geo.Cylinders.QuadPart 
			  , (unsigned)geo.TracksPerCylinder
			  , (unsigned)geo.SectorsPerTrack  
			  , (unsigned)geo.BytesPerSector   
			);
			PutPos(8, "Cylinder/Track/Sector:Size"
			  , COLUMN2 + 10 - strlen(t1), t1
			  , COLUMN3                  , Fmt(t2, " (%s)", FmtI64(t3, geo.Cylinders.QuadPart * geo.TracksPerCylinder * geo.SectorsPerTrack * geo.BytesPerSector))
			  , 0);
		}
	}
#endif
#if	defined(LINUX)
	if ( this->devCtl.GetHType() == DevCtl::HT_BLKDEV
	  || this->devCtl.GetHType() == DevCtl::HT_CDROM
	 )
	{
		struct hd_driveid		drvid;
	//	struct hd_big_geometry	bgeo;
	//	struct hd_geometry		geo;
		char					*t;

		memset( &drvid, 0, sizeof(drvid));
		if ( ioctl(this->devCtl.GetHandle(), HDIO_GET_IDENTITY, &drvid) == 0 )
		{
			PutPos(4        , "HDIO GET IDENTITY", 0);
			if ( drvid.cyls || drvid.heads || drvid.sectors )
			{
				sprintf(t1, "%u/%u/%u", (unsigned)drvid.cyls, (unsigned)drvid.heads, (unsigned)drvid.sectors);
				PutPos(8, "Physical Cylinder/Heads/Sectors", COLUMN2-6, Fmt(t2,"%16s",t1), 0);
			}
			if ( drvid.track_bytes  ) PutPos(8, "Unformatted Track size" , COLUMN2, Fmt(t1, "%10u", (unsigned)drvid.track_bytes ) , 0);
			if ( drvid.sector_bytes ) PutPos(8, "Unformatted Sector size", COLUMN2, Fmt(t1, "%10u", (unsigned)drvid.sector_bytes) , 0);
			if ( drvid.serial_no[0] ) PutPos(8, "Serial No."             , COLUMN2-10, Fmt(t1, "%20.20s", drvid.serial_no       ) , 0);
			if ( drvid.ecc_bytes    ) PutPos(8, "ECC Bytes"              , COLUMN2, Fmt(t1, "%10u", (unsigned)drvid.ecc_bytes   ) , 0);
			if ( drvid.fw_rev[0]    ) PutPos(8, "F/W Revision"           , COLUMN2+ 2, Fmt(t1, "%8.8s", drvid.fw_rev            ) , 0);
			if ( drvid.model[0]     ) PutPos(8, "Model"                  , COLUMN2-30, Fmt(t1, "%40.40s", drvid.model           ) , 0);
			switch ( drvid.tPIO )
			{
				case 0 : t = "slow"  ; break;
				case 1 : t = "medium"; break;
				case 2 : t = "fast"  ; break;
				default: sprintf(t1, "%02X", (unsigned)drvid.tPIO); t = t1; break;
			}
			PutPos(8, "PIO"                  , COLUMN2, Fmt(t1, "%10.10s", t) , 0);
			switch ( drvid.tDMA )
			{
				case 0 : t = "slow"  ; break;
				case 1 : t = "medium"; break;
				case 2 : t = "fast"  ; break;
				default: sprintf(t1, "%02X", (unsigned)drvid.tDMA); t = t1; break;
			}
			PutPos(8, "DMA"                  , COLUMN2, Fmt(t1, "%10.10s", t) , 0);
			if ( drvid.cur_cyls || drvid.cur_heads || drvid.cur_sectors )
			{
				sprintf(t1, "%u/%u/%u", (unsigned)drvid.cur_cyls, (unsigned)drvid.cur_heads, (unsigned)drvid.cur_sectors);
				PutPos(8, "Logical Cylinder/Heads/Sectors", COLUMN2-6, Fmt(t2,"%16s",t1), 0);
			}
			if ( drvid.lba_capacity ) PutPos(8, "LBA Capacity"           , COLUMN2, Fmt(t1, "%10u", (unsigned)drvid.lba_capacity) , 0);
		}
	}
#endif

	if ( devParam.inqData->unknownId == FALSE )
	{
		if ( this->devCtl.ScsiTestUnitReady(pathId, targetId, lun) == 0 )
			PutFmt(4, "Test Unit Ready", "OK", "");

		this->PrintDevInfo_vpd(devParam);
		this->PrintDevInfo_modeSense(devParam);
		this->DevTest::PrintDevInfo_defectData(devParam);
	}
	return 0;
}

int		DevTest::PrintDevInfo_vpd(DevTest::DevParam & devParam)
{
	unsigned	pathId, targetId, lun;
	BYTE		buffer[255], codelist[255], map[256];
	BOOL		list_n = 0;
	unsigned	page, len;
	int			mode = 0;
	enum map_enu { MAP_OFF, MAP_DISP, MAP_ERR };

	if ( devParam.inqData == NULL )
		return -1;
	pathId   = devParam.inqData->GetPathId();
	targetId = devParam.inqData->GetTargetId();
	lun      = devParam.inqData->GetLun();

	memset(map, MAP_OFF, sizeof(map));
	page = 0x00;
	for (;;)
	{
		if ( this->devCtl.ScsiInquiry(pathId, targetId, lun, page, sizeof(buffer), buffer) < 0 )
		{	// error
			break;
		}
		len = buffer[3];
		if ( map[buffer[1]] != MAP_DISP )
		{	// $BL$I=<((B
			char	msg[80];

			//  ----+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8
			// "    VPD:Page Code List               ##########"
			// "                                     ####: ## ## ## ## ## ## ## ##
			sprintf(msg, "VPD:Page Code %02XH", buffer[1]);
			if ( len > 4 )
				this->PrintDumpOct(msg, TRUE, len - 4, &buffer[4]);
		}

		if ( map[0x00] != MAP_DISP )
		{	// No Get page code list
			if ( buffer[1] == 0x00 )
			{	// page code list
				list_n = 4;
				memcpy(codelist, buffer, sizeof(codelist));
				map[0x00] = MAP_DISP;
				page = codelist[list_n];
			} else
			{	// Request page code list, bad other page
				break;
			}
		} else
		{	// Get page code list
			if ( map[buffer[1]] != MAP_DISP )
			{	// $BL$I=<((B
				map[buffer[1]] = MAP_DISP;
			}
			list_n++;
			if ( list_n >= codelist[3] )
				break;
			page = codelist[list_n];
		}
	}

	return 0;
}

int		DevTest::PrintDevInfo_modeSense(DevTest::DevParam & devParam)
{
	unsigned	pathId, targetId, lun;
	BYTE		buffer[255], *p;

	if ( devParam.inqData == NULL )
		return -1;
	pathId   = devParam.inqData->GetPathId();
	targetId = devParam.inqData->GetTargetId();
	lun      = devParam.inqData->GetLun();

	if ( this->devCtl.ScsiModeSense(pathId, targetId, lun, 0x3F, sizeof(buffer), buffer) < 0 )
		return -1;
	if ( buffer[0] == 0 )
		return 0;
	if ( this->opt.dumpModeSense )
		this->PrintDumpOct("modeSense", TRUE, buffer[0] - 2, buffer);
	p = &buffer[4];
	if ( buffer[3] > 0 )
	{	// block descripter
		BYTE				*data;
		char				text[80];

		//  ----+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8
		// "    modeSense: Block Descripter ##H  (########) ##########"
		// "               %-33s %10u\n", "Write Protect", (buffer[2]>>7) & 1);
		sprintf(text, "Block Descripter %02XH", p[0]);
		fprintf(stdout, "    modeSense: %-22s (%8u) %10u\n"
		  , text
		  , ((p[5]<<16) + (p[6]<<8) + p[7])
		  , ((p[1]<<16) + (p[2]<<8) + p[3])
		);
		if ( (data = devParam.inqData->GetData()) != NULL )
		{
			switch ( data[1] & 0x3F )
			{
				case 0x00:	// direct access device
					fprintf(stdout, "               %-33s %10u\n", "Write Protect", (buffer[2]>>7) & 1);
					fprintf(stdout, "               %-33s %10u\n", "Disable PageOut/Force Unit Access", (buffer[2]>>4) & 1);
					break;
			}
		}
		p += buffer[3];
	}
	while ( p < buffer + buffer[0] - 2 )
	{
		char	msg[80], s1[80], t1[80];

		if ( p[1] == 0 )
			break;
		//  ----+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8
		// "    modeSense: pagecode ##H size:%3u - Disconnect/Reconnect"
		sprintf(msg, "modeSense: pagecode %02XH size:%3u", p[0] & 0x3F, p[1]);
		switch ( p[0] & 0x3F )
		{
			case 0x01:	// Read/Write Error Recovery Parameter
			{
				//  ----+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8
				// "    modeSense: pagecode ##H - Disconnect/Reconnect"
				// "    <-----------------------------------------> ##########"
				if ( p[1] < 10 )
					goto ERR;
				fprintf(stdout, "    %-33s - Read/Write Error Recovery Parameter\n", msg);
				PutPos(15, "Automatic Write Reallocation Enabled" , COLUMN2+5, Fmt(t1,"%5s", (p[2] & 0x80) ? "ON" : "OFF"), 0);
				PutPos(15, "Automatic Read  Reallocation Enabled" , COLUMN2+5, Fmt(t1,"%5s", (p[2] & 0x40) ? "ON" : "OFF"), 0);
				PutPos(15, "Transfer Block"                       , COLUMN2+5, Fmt(t1,"%5s", (p[2] & 0x20) ? "ON" : "OFF"), 0);
				PutPos(15, "Read Continuous"                      , COLUMN2+5, Fmt(t1,"%5s", (p[2] & 0x10) ? "ON" : "OFF"), 0);
				PutPos(15, "Enable Early Recovery"                , COLUMN2+5, Fmt(t1,"%5s", (p[2] & 0x08) ? "ON" : "OFF"), 0);
				PutPos(15, "Post Error"                           , COLUMN2+5, Fmt(t1,"%5s", (p[2] & 0x04) ? "ON" : "OFF"), 0);
				PutPos(15, "Disable Tranfer on Error"             , COLUMN2+5, Fmt(t1,"%5s", (p[2] & 0x02) ? "ON" : "OFF"), 0);
				PutPos(15, "Disable Correction"                   , COLUMN2+5, Fmt(t1,"%5s", (p[2] & 0x01) ? "ON" : "OFF"), 0);
				PutPos(15, "Read Retry Count"                     , COLUMN2  , Fmt(t1,"%10u", p[3]), 0);
				PutPos(15, "Correction Span"                      , COLUMN2  , Fmt(t1,"%10u", p[4]), 0);
				PutPos(15, "Head Offset"                          , COLUMN2  , Fmt(t1,"%10i", (int)((signed char)p[5])), 0);
				PutPos(15, "Data Strobe Offset"                   , COLUMN2  , Fmt(t1,"%10i", (int)((signed char)p[6])), 0);
				PutPos(15, "Write Retry Count"                    , COLUMN2  , Fmt(t1,"%10u", p[8]), 0);
				PutPos(15, "Recovey Time Limit"                   , COLUMN2  , Fmt(t1,"%10u ms", (p[10]<<8)+p[11]), 0);
				break;
			}
			case 0x02:
			{
				if ( p[1] < 14 )
					goto ERR;
				fprintf(stdout, "    %-33s - Disconnect/Reconnect\n", msg);
				fprintf(stdout, "               %-33s %10u\n", "Buffer Full Ratio"    , p[2]);
				fprintf(stdout, "               %-33s %10u\n", "Buffer Empty Ratio"   , p[3]);
				fprintf(stdout, "               %-33s %10u\n", "Bus Inactivity Limit" , (p[4]<<8) + p[5]);
				fprintf(stdout, "               %-33s %10u\n", "Disconnect Time Limit", (p[6]<<8) + p[7]);
				fprintf(stdout, "               %-33s %10u\n", "Connect Time Limit"   , (p[8]<<8) + p[9]);
				fprintf(stdout, "               %-33s %10u\n", "Maximum Burst Size"   , (p[10]<<8) + p[11]);
				fprintf(stdout, "               %-33s %10X\n", "Data Transfer Disconnect control", p[12] & 3);
				break;
			}
			case 0x04:	// Drive Parameter
			{
				static const char * rpl[] =
				{
					"Sync: disable or not supported",
					"Sync: slave",
					"Sync: master",
					"Sync: master control",
				};
				if ( p[1] < 22 )
					goto ERR;
				PutPos(4,msg, 38," - Drive Parameter", 0);
				PutPos(15, "Number of Cylinders"                    , COLUMN2  , Fmt(t1,"%10u", (p[2]<<16)+(p[3]<<8)+p[4]), 0);
				PutPos(15, "Number of Heads"                        , COLUMN2  , Fmt(t1,"%10u", p[5]), 0);
				PutPos(15, "Starting Cylinder-Write Precompensation", COLUMN2  , Fmt(t1,"%10u", (p[6]<<16)+(p[7]<<8)+p[8]), 0);
				PutPos(15, "Starting Cylinder-Reduced Write Current", COLUMN2  , Fmt(t1,"%10u", (p[9]<<16)+(p[10]<<8)+p[11]), 0);
				PutPos(15, "Drive Step Rate"                        , COLUMN2  , Fmt(t1,"%10u", (p[12]<<8)+p[13]), 0);
				PutPos(15, "Landing Zone Cylinder"                  , COLUMN2  , Fmt(t1,"%10u", (p[14]<<16)+(p[15]<<8)+p[16]), 0);
				PutPos(15, "Rotational Position Locking"            , COLUMN2  , Fmt(t1,"%10u", p[17] & 3), COLUMN2 + 11, rpl[p[17]&3], 0);
				PutPos(15, "Rotational offset"                      , COLUMN2  , Fmt(t1,"%10i", (int)((signed char)p[18])), 0);
				PutPos(15, "Medium rotation rate"                   , COLUMN2  , Fmt(t1,"%10u", (p[20]<<8)+p[21]), 0);
				break;
			}
			case 0x08:	// Cache Control Parameter
			{
				if ( p[1] < 10 )
					goto ERR;
				fprintf(stdout, "    %-33s - Cache Control Parameter\n", msg);
				fprintf(stdout, "               %-33s %10u\n", "Write Cache Enable", (p[2]>>2) & 1);
				fprintf(stdout, "               %-33s %10u\n", "Multiplication Factor", (p[2]>>1) & 1);
				fprintf(stdout, "               %-33s %10u\n", "Read Cache Disable", (p[2]) & 1);
				fprintf(stdout, "               %-33s %10X\n", "Demand Read Retention Priority", (p[3]>>4) & 0xF);
				fprintf(stdout, "               %-33s %10X\n", "Write Retention Priority", (p[3]) & 0xF);
				fprintf(stdout, "               %-33s %10u\n", "Disable Pre-fetch Transfer Length", (p[4]<<8) + p[5]);
				fprintf(stdout, "               %-33s %10u\n", "Minimum Pre-fetch", (p[6]<<8) + p[7]);
				fprintf(stdout, "               %-33s %10u\n", "Maximum Pre-fetch", (p[8]<<8) + p[9]);
				fprintf(stdout, "               %-33s %10u\n", "Maximum Pre-fetch Ceiling", (p[10]<<8) + p[11]);
				break;
			}
			case 0x09:	// Periphery Device Parameter
			{
				if ( p[1] < 7 )
					goto ERR;
				fprintf(stdout, "    %-33s - Periphery Device Parameter\n", msg);
				sprintf(s1,"%04X", (p[2]<<8) + p[3]);
				fprintf(stdout, "               %-33s %10s\n", "interface ID", s1);
				this->PrintDumpOct("          vender depended", TRUE, p[1]-6, &p[8]);
				break;
			}
			case 0x0A: // Control Mode Parameter
			{
				if ( p[1] < 6 )
					goto ERR;
				fprintf(stdout, "    %-33s - Control Mode Parameter\n", msg);
				fprintf(stdout, "               %-33s %10s\n", "Report Log Exception Condition", (p[2] & 1) ? "ON" : "OFF");
				sprintf(s1,"%1XH", (p[3]>>4) & 7);
				fprintf(stdout, "               %-33s %10s\n", "Queue Algorithm modfier", s1);
				fprintf(stdout, "               %-33s %10u\n", "Queue Error Management", (p[3]>>1) & 1);
				fprintf(stdout, "               %-33s %10u\n", "Disable Queuing", p[3] & 1);
				fprintf(stdout, "               %-33s %10u\n", "Enable ECA", (p[4]>>7) & 1);
				fprintf(stdout, "               %-33s %10u\n", "Ready AEN Permission", (p[4]>>2) & 1);
				fprintf(stdout, "               %-33s %10u\n", "Unit Attention AEN Permission", (p[4]>>1) & 1);
				fprintf(stdout, "               %-33s %10u\n", "Error AEN Permission", (p[4]>>1) & 1);
				fprintf(stdout, "               %-33s %10u\n", "Ready AEN Holdoff Time", (p[6]<<8) + p[7]);
				break;
			}
			case 0x0B:	// Media Type Parameter
			{
				if ( p[1] < 6 )
					goto ERR;
				PutPos(4,msg, 38," - Media Type Parameter", 0);	// XXH XXH XXH XXH
				PutPos(15, "Media Type 1-4", COLUMN2-5, Fmt(t1,"%02XH %02XH %02XH %02XH", p[4],p[5],p[6],p[7]), 0);
				break;
			}
			case 0x0C:	// Notch/Partition Parameter
			{
				if ( p[1] < 22 )
					goto ERR;
				PutPos(4,msg, 38," - Notch/Partition Parameter", 0);
				PutPos(15, "Nocthed Device"            , COLUMN2, Fmt(t1,"%10s", (p[2] & 0x80) ? "ON" : "OFF"), 0);
				PutPos(15, "Logical or Physical Notch" , COLUMN2, Fmt(t1,"%10s", (p[2] & 0x40) ? "LOGICAL" : "PHYSICAL"), 0);
				PutPos(15, "Maximum Number of Notches" , COLUMN2, Fmt(t1,"%10u", (p[4]<<8)+p[5]), 0);
				PutPos(15, "Active Notche"             , COLUMN2, Fmt(t1,"%10u", (p[6]<<8)+p[7]), 0);
				PutPos(15, "Starting Boundary"         , COLUMN2, Fmt(t1,"%10u", (p[8]<<24)+(p[9]<<16)+(p[10]<<8)+p[11]), 0);
				PutPos(15, "Ending boundary"           , COLUMN2, Fmt(t1,"%10u", (p[12]<<24)+(p[13]<<16)+(p[14]<<8)+p[15]), 0);
				PutPos(15, "Pages Notched Parameter"   , COLUMN2-6, Fmt(t1,"%02X%02X%02X%02X%02X%02X%02X%02X",p[16],p[17],p[18],p[19],p[20],p[21],p[22],p[23]), 0);
				break;
			}
			default:
			{
			ERR:
				this->PrintDumpOct(msg, TRUE, p[1], p+2);
				break;
			}
		}
		p += (unsigned)p[1] + 2;
	}

	return 0;
}

int		DevTest::PrintDevInfo_defectData(DevTest::DevParam & devParam)
{
	unsigned	pathId, targetId, lun;
	BYTE		buffer[4096];
	int			list;

	if ( devParam.inqData == NULL )
		return -1;
	pathId   = devParam.inqData->GetPathId();
	targetId = devParam.inqData->GetTargetId();
	lun      = devParam.inqData->GetLun();
	for ( list = 0; list < 2; list++ )
	{
		int			len, n;
		char		*textList, t1[80];
		BYTE		*p;

		if ( this->devCtl.ScsiReadDefectData(pathId, targetId, lun, 1<<list, 0, sizeof(buffer), buffer) < 0 )
			return -1;
		len = (buffer[2]<<8) + buffer[3];
		switch ( (buffer[1] >> 3) & 3 )
		{
			case  1: textList = "G List"    ; break;
			case  2: textList = "P List"    ; break;
			case  3: textList = "G & P List"; break;
			default: textList = ""          ; break;
		}
		//  ----+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8
		// "    Defect List (Block adress)                  G & P List "
		// "    <------------------------------- ########## ########## "
		p = &buffer[4];
		n = 0;
		switch ( buffer[1] & 7 )	// type
		{
			case 0:	// block address format
				sprintf(t1, "Defect List (Block Address)");
				fprintf(stdout, "    %-43s %s\n", t1, textList);
				while ( len > 0 )
				{
					fprintf(stdout,"    ");
					fprintf(stdout,"%02X%02X%02X%02X",
					    p[0], p[1], p[2], p[3]);
					if ( (n % 6) == 5 )
						fprintf(stdout,"\n");
					if ( len < 4 )
						break;
					++n;
					len -= 4;
					p += 4;
				}
				if ( n == 0 )
					fprintf(stdout,"        %s\n", "nothing");
				else if ( n % 6 )
					fprintf(stdout,"\n");
				break;
			case 4:	// index byte format
				sprintf(t1, "Defect List (Distance from index)");
				fprintf(stdout, "    %-43s %s\n", t1, textList);
				while ( len > 0 )
				{
					fprintf(stdout,"    ");
					fprintf(stdout,"Cy:%02X%02X%02X Hd:%02X Bt:%02X%02X%02X%02X",
					    p[0], p[1], p[2], p[3], p[4], p[5], p[6], p[7]);
					if ( (n % 3) == 2 )
						fprintf(stdout,"\n");
					if ( len < 8 )
						break;
					++n;
					len -= 8;
					p += 8;
				}
				if ( n == 0 )
					fprintf(stdout,"        %s\n", "nothing");
				else if ( n % 3 )
					fprintf(stdout,"\n");
				break;
			case 5:	// physical sector format
				sprintf(t1, "Defect List (Physical Sector Address)");
				fprintf(stdout, "    %-43s %s\n", t1, textList);
				while ( len > 0 )
				{
					fprintf(stdout,"    ");
					fprintf(stdout,"Cy:%02X%02X%02X Hd:%02X No:%02X%02X%02X%02X",
					    p[0], p[1], p[2], p[3], p[4], p[5], p[6], p[7]);
					if ( (n % 3) == 2 )
						fprintf(stdout,"\n");
					if ( len < 8 )
						break;
					++n;
					len -= 8;
					p += 8;
				}
				if ( n == 0 )
					fprintf(stdout,"        %s\n", "nothing");
				else if ( n % 3 )
					fprintf(stdout,"\n");
				break;
			case 6:	// vender unique
			default:
				sprintf(t1, "Defect List (Vender Unique)");
				fprintf(stdout, "    %-43s %s\n", t1, textList);
				if ( len == 0 )
					fprintf(stdout,"        %s\n", "nothing");
				else
					this->PrintDumpOct("", TRUE, len, p);
				break;
		}
	}
	return 0;
}
