/////////////////////////////////////////////////////////////////////////
//	"devtest_lib.cpp"
//------------------------------------------------------------------------
//	$Id: devtest_lib.cpp 50 2003-12-11 01:39:41Z  $
//////////////////////////////////////////////////////////////////////////

#include	<stdlib.h>
#include	<stdio.h>
#include	<stdarg.h>
#include	<string.h>
#include	<ctype.h>
#include	<time.h>
#if	defined(LINUX)
# include	<sys/time.h>
#endif
#include	"devtest.h"
#include	"defs.h"

char *	DevTest::FmtI64(char * buffer, INT64 n)
{
	unsigned	len;
	char		*src, *dst, tmp[32];

	sprintf(tmp,"%"FI64"i", n);
	if ( (len = strlen(tmp)) == 0 )
	{
		buffer[0] = '0';
		buffer[1] = '\0';
		return buffer;
	}
	src = (char *)(tmp + len - 1);
	dst = (char *)(buffer + len + ((len-1) / 3));
	*dst-- = '\0';

	{
		unsigned	i = 0;

		for (;;)
		{
			*dst-- = *src--;
			if ( ++i >= len )
				break;
			if ( (i%3) == 0 )
				*dst-- = ',';
		}
	}
	return buffer;
}

char *	DevTest::Fmt(char * buffer, const char * form, ... )
{
	va_list	arg;

	va_start(arg, form);
	vsprintf(buffer, form, arg);
	va_end(arg);
	return buffer;
}

//  ----+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8
// "### [## ## ##]-[## ## ##] ########## ########## ##:#################### #######
// "                                     ########## ########## -------------------
void	DevTest::PutFmt(int spc, const char * text1, const char * val, const char *text2)
{
	fprintf(stdout, "%*s%-*s %10s %s\n"
	  ,spc, ""
	  ,48 - spc, text1
	  ,val
	  ,text2
	);
}

void	DevTest::PutPos(int pos, ... )
{
	va_list	arg;
	char *	text;
	int		x = 0, len;

	va_start(arg, pos);
	while ( pos > 0 )
	{
		text = va_arg(arg, char*);
		len = strlen(text);
		if ( pos > x )
		{
			fprintf(stdout, "%*s", pos - x, "");
			x = pos;
		}
		fprintf(stdout, "%s", text);
		x += len;
		pos  = va_arg(arg, int);
	}
	fprintf(stdout,"\n");
	va_end(arg);
}


void		DevTest::PrintDumpOct(const char * msg, int ascii, unsigned size, BYTE * buffer)
{
	unsigned	n;

	if ( size == 0 )
		return;
	//  ----+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8
	// "    #################################  ####: ## ## ## ## ## ## ## ##  ########
	fprintf(stdout,"    %-33s  %04X:", msg, 0 );
	for ( n = 0; n < ((size + 8)/8)*8; n++ )
	{
		if ( n > 0 && (n % 8) == 0 )
			fprintf(stdout,"    %-33s  %04X:", "", n );
		if ( n < size )
			fprintf(stdout, " %02X", buffer[n]);
		else
			fprintf(stdout, "   ");
		if ( (n % 8) == 7 )
		{
			if ( ascii )
			{
				unsigned	i, i8;

				fprintf(stdout, "  ");
				i8 = (n/8)*8;
				for ( i = i8; i < size && i < i8+8; i++ )
					fprintf(stdout, "%c", isprint(buffer[i]) ? buffer[i] : '.');
			}
			fprintf(stdout, "\n");
			if ( n + 1 >= size )
				break;
		}
	}
}

void		DevTest::PrintDumpHex(unsigned addr, unsigned blockSize, unsigned size, BYTE * buffer)
{
	unsigned	x, y;

	if ( size == 0 )
		return;
	//  ----+----1----+----2----+----3----+----4----+----5----+----6----+----7----+----8
	// "ADDRESS    +0 +1 +2 +3 +4 +5 +6 +7 - +8 +9 +A +B +C +D +E +F  ----------------
	// "XXXX-XXXX  ## ## ## ## ## ## ## ## - ## ## ## ## ## ## ## ##  ################
	fprintf(stdout,"ADDRESS    +0 +1 +2 +3 +4 +5 +6 +7 - +8 +9 +A +B +C +D +E +F  ----------------\n");
	for ( y = 0; y < size; y += 16 )
	{
		fprintf(stdout,"%04X-%04X ", (addr+y)>>16, (addr+y) & 0xFFFF);
		for ( x = 0; x < 16; x++ )
		{
			unsigned	c;

			if ( y + x < size )
			{
				c = buffer[y + x];
				fprintf(stdout, " %02X", c);
			} else
				fprintf(stdout, "   ");
			if ( x == 7 )
				fprintf(stdout," -");
		}
		fprintf(stdout,"  ");
		for ( x = 0; x < 16; x++ )
		{
			unsigned	c;

			if ( y + x < size )
			{
				c = buffer[y + x];
				c = isprint(c) ? c : '.';
			} else
				c= ' ';
			fprintf(stdout,"%c", c);
		}
		fprintf(stdout,"\n");
	}
}

char *	DevTest::FormatSize(char * buffer, INT64 size)
{
	unsigned	n, m;
	INT64		ub;

	switch ( this->opt.byteUnit )
	{
		case BYTEUNIT_210:
			ub = I64(1024);
			break;
		case BYTEUNIT_103:
		default:
			ub = I64(1000);
			break;
	}

	if ( size < ub )
	{
		sprintf(buffer, "%4u.0B", (unsigned)size);
	} else if ( size < ub * ub )
	{	// Kilo
		n = (unsigned)(size / ub);
		m = (unsigned)(I64(10) * (size % ub) / ub);
		sprintf(buffer, "%4u.%1u%c", n, m, (this->opt.byteUnit == BYTEUNIT_210 ? 'K' : 'k'));
	} else if ( size < ub * ub * ub )
	{	// Mega
		n = (unsigned)(size / (ub*ub));
		m = (unsigned)(I64(10) * (size % (ub*ub)) / (ub*ub));
		sprintf(buffer, "%4u.%1uM", n, m);
	} else if ( size < ub * ub * ub * ub )
	{	// Giga
		n = (unsigned)(size / (ub*ub*ub));
		m = (unsigned)(I64(10) * (size % (ub*ub*ub)) / (ub*ub*ub));
		sprintf(buffer, "%4u.%1uG", n, m);
	} else if ( size < ub * ub * ub * ub * ub )
	{	// Tela
		n = (unsigned)(size / (ub*ub*ub*ub));
		m = (unsigned)(I64(10) * (size % (ub*ub*ub*ub)) / (ub*ub*ub*ub));
		sprintf(buffer, "%4u.%1uT", n, m);
	}

	return buffer;
}

char *	DevTest::FormatTimeRemain(char * buffer, unsigned sec)
{
	if ( sec < 60 )
		sprintf(buffer, "         %02us", sec);
	else if ( sec < 5 * 60 )
		sprintf(buffer, "     %02um %02us", sec/60, sec%60);
	else if ( sec < 60 * 60 )
		sprintf(buffer, "     %02um    ", sec/60 );
	else
		sprintf(buffer, "%3uh %02um    ", sec/60/60, (sec/60)%60);
	return buffer;
}

char *	DevTest::FormatTime(char * buffer, unsigned sec)	// for devtest_udf.lib
{
#if 0
	if ( sec < 60 )
		sprintf(buffer, "%02us", sec);
	else if ( sec < 60 * 60)
		sprintf(buffer, "%2um %02us", sec/60, sec%60);
	else if ( sec < 24 * 60 * 60 )
		sprintf(buffer, "%2uh %02um %02us", sec/60/60, (sec/60)%60, sec%60 );
	else
		sprintf(buffer, "%3ud %02uh %02um %02us", sec/60/60/24, (sec/60/60)%24, (sec/60)%60, sec%60);
#else
	sprintf(buffer, "%d %02u:%02u:%02u", sec/60/60/24, (sec/60/60)%24, (sec/60)%60, sec%60);
#endif
	return buffer;
}

#if	defined(WIN32)
char *	DevTest::GetErrMsg(int errcode)
{
	char	*msgbuf;
	int		ret;

	ret = ::FormatMessage(
	    FORMAT_MESSAGE_ALLOCATE_BUFFER |
	    FORMAT_MESSAGE_FROM_SYSTEM |
	    FORMAT_MESSAGE_IGNORE_INSERTS,
	    NULL,
	    errcode,
	    MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), // $B4{Dj$N8@8l(B
	    (LPTSTR) &msgbuf,
	    0,
	    NULL
		);
	return ret > 0 ? msgbuf : "";
}
#endif

char *	DevTest::GetPartitionType(unsigned type)
{
	static const char *tbl[] = 
	{
	/* 0x00 */	"Empty",
	/* 0x01 */	"FAT12",
	/* 0x02 */	"XENIX root",
	/* 0x03 */	"XENIX user",
	/* 0x04 */	"FAT16 (<32M)",
	/* 0x05 */	"Extended",
	/* 0x06 */	"FAT16 (>=32M)",
	/* 0x07 */	"NTFS",
	/* 0x08 */	"OS/2",
	/* 0x09 */	"AIX data",
	/* 0x0A */	"OS/2 Boot Manager",
	/* 0x0B */	"FAT32",
	/* 0x0C */	"FAT32 LBA",
	/* 0x0D */	"",
	/* 0x0E */	"FAT16 LBA",
	/* 0x0F */	"Extended LBA",

	/* 0x10 */	"",
	/* 0x11 */	"Hidden FAT12",
	/* 0x12 */	"",
	/* 0x13 */	"",
	/* 0x14 */	"Hidden FAT16 (<32M)",
	/* 0x15 */	"",
	/* 0x16 */	"Hidden FAT16 (>=32M)",
	/* 0x17 */	"Hidden HPFS",
	/* 0x18 */	"",
	/* 0x19 */	"",
	/* 0x1A */	"",
	/* 0x1B */	"Hidden FAT32 Hidden",
	/* 0x1C */	"Hidden FAT32 LBA",
	/* 0x1D */	"",
	/* 0x1E */	"Hidden FAT16 LBA",
	/* 0x1F */	"",

	/* 0x20 */	"",
	/* 0x21 */	"",
	/* 0x22 */	"",
	/* 0x23 */	"",
	/* 0x24 */	"NEC DOS 3.x",
	/* 0x25 */	"",
	/* 0x26 */	"",
	/* 0x27 */	"",
	/* 0x28 */	"",
	/* 0x29 */	"",
	/* 0x2A */	"",
	/* 0x2B */	"",
	/* 0x2C */	"",
	/* 0x2D */	"",
	/* 0x2E */	"",
	/* 0x2F */	"",

	/* 0x30 */	"",
	/* 0x31 */	"",
	/* 0x32 */	"",
	/* 0x33 */	"",
	/* 0x34 */	"",
	/* 0x35 */	"",
	/* 0x36 */	"",
	/* 0x37 */	"",
	/* 0x38 */	"",
	/* 0x39 */	"Paln9",
	/* 0x3A */	"",
	/* 0x3B */	"",
	/* 0x3C */	"PartitionMagic",
	/* 0x3D */	"Hidden NetWare",
	/* 0x3E */	"",
	/* 0x3F */	"",

	/* 0x40 */	"Venix 80286",
	/* 0x41 */	"Linux/MINIX",
	/* 0x42 */	"Linux swap",
	/* 0x43 */	"Linux native",
	/* 0x44 */	"GoBack",
	/* 0x45 */	"",
	/* 0x46 */	"",
	/* 0x47 */	"",
	/* 0x48 */	"",
	/* 0x49 */	"",
	/* 0x4A */	"",
	/* 0x4B */	"",
	/* 0x4C */	"",
	/* 0x4D */	"",
	/* 0x4E */	"",
	/* 0x4F */	"",

	/* 0x50 */	"",
	/* 0x51 */	"",
	/* 0x52 */	"CP/M",
	/* 0x53 */	"",
	/* 0x54 */	"",
	/* 0x55 */	"EZ-Drive",
	/* 0x56 */	"",
	/* 0x57 */	"",
	/* 0x58 */	"",
	/* 0x59 */	"",
	/* 0x5A */	"",
	/* 0x5B */	"",
	/* 0x5C */	"",
	/* 0x5D */	"",
	/* 0x5E */	"",
	/* 0x5F */	"",

	/* 0x60 */	"",
	/* 0x61 */	"",
	/* 0x62 */	"",
	/* 0x63 */	"",
	/* 0x64 */	"Novell NetWare 286",
	/* 0x65 */	"Novell NetWare 386",
	/* 0x66 */	"Novell NetWare SMS",
	/* 0x67 */	"Novell",
	/* 0x68 */	"Novell",
	/* 0x69 */	"Novell",
	/* 0x6A */	"",
	/* 0x6B */	"",
	/* 0x6C */	"",
	/* 0x6D */	"",
	/* 0x6E */	"",
	/* 0x6F */	"",

	/* 0x70 */	"",
	/* 0x71 */	"",
	/* 0x72 */	"",
	/* 0x73 */	"",
	/* 0x74 */	"",
	/* 0x75 */	"IBM PC/IX",
	/* 0x76 */	"",
	/* 0x77 */	"",
	/* 0x78 */	"",
	/* 0x79 */	"",
	/* 0x7A */	"",
	/* 0x7B */	"",
	/* 0x7C */	"",
	/* 0x7D */	"",
	/* 0x7E */	"",
	/* 0x7F */	"",

	/* 0x80 */	"MINIX",
	/* 0x81 */	"MINUX",
	/* 0x82 */	"Linux swap/Solaris86",
	/* 0x83 */	"Linix native",
	/* 0x84 */	"Hidden partition",
	/* 0x85 */	"Linux extended",
	/* 0x86 */	"NTFS volume set",
	/* 0x87 */	"",
	/* 0x88 */	"",
	/* 0x89 */	"",
	/* 0x8A */	"",
	/* 0x8B */	"",
	/* 0x8C */	"",
	/* 0x8D */	"",
	/* 0x8E */	"",
	/* 0x8F */	"",

	/* 0x90 */	"",
	/* 0x91 */	"",
	/* 0x92 */	"",
	/* 0x93 */	"",
	/* 0x94 */	"",
	/* 0x95 */	"",
	/* 0x96 */	"",
	/* 0x97 */	"",
	/* 0x98 */	"",
	/* 0x99 */	"",
	/* 0x9A */	"",
	/* 0x9B */	"",
	/* 0x9C */	"",
	/* 0x9D */	"",
	/* 0x9E */	"",
	/* 0x9F */	"BSD/OS",

	/* 0xA0 */	"",
	/* 0xA1 */	"",
	/* 0xA2 */	"",
	/* 0xA3 */	"",
	/* 0xA4 */	"",
	/* 0xA5 */	"BSD/386",
	/* 0xA6 */	"OpenBSD",
	/* 0xA7 */	"NEXTSTEP",
	/* 0xA8 */	"Mac OS-X",
	/* 0xA9 */	"NetBSD",
	/* 0xAA */	"",
	/* 0xAB */	"Mac OS-X boot",
	/* 0xAC */	"",
	/* 0xAD */	"",
	/* 0xAE */	"",
	/* 0xAF */	"",

	/* 0xB0 */	"",
	/* 0xB1 */	"",
	/* 0xB2 */	"",
	/* 0xB3 */	"",
	/* 0xB4 */	"",
	/* 0xB5 */	"",
	/* 0xB6 */	"NT mirror FAT16",
	/* 0xB7 */	"NT mirror NTFS",
	/* 0xB8 */	"",
	/* 0xB9 */	"",
	/* 0xBA */	"",
	/* 0xBB */	"",
	/* 0xBC */	"",
	/* 0xBD */	"",
	/* 0xBE */	"",
	/* 0xBF */	"",

	/* 0xC0 */	"",
	/* 0xC1 */	"",
	/* 0xC2 */	"Hidden Linux",
	/* 0xC3 */	"Hidden Linux swap",
	/* 0xC4 */	"",
	/* 0xC5 */	"",
	/* 0xC6 */	"NT FAT vol/stripe",
	/* 0xC7 */	"NT NTFS vol/stripe",
	/* 0xC8 */	"",
	/* 0xC9 */	"",
	/* 0xCA */	"",
	/* 0xCB */	"",
	/* 0xCC */	"",
	/* 0xCD */	"",
	/* 0xCE */	"",
	/* 0xCF */	"",

	/* 0xD0 */	"",
	/* 0xD1 */	"",
	/* 0xD2 */	"",
	/* 0xD3 */	"",
	/* 0xD4 */	"",
	/* 0xD5 */	"",
	/* 0xD6 */	"",
	/* 0xD7 */	"",
	/* 0xD8 */	"",
	/* 0xD9 */	"",
	/* 0xDA */	"",
	/* 0xDB */	"",
	/* 0xDC */	"",
	/* 0xDD */	"",
	/* 0xDE */	"",
	/* 0xDF */	"",

	/* 0xE0 */	"",
	/* 0xE1 */	"",
	/* 0xE2 */	"",
	/* 0xE3 */	"",
	/* 0xE4 */	"",
	/* 0xE5 */	"",
	/* 0xE6 */	"",
	/* 0xE7 */	"",
	/* 0xE8 */	"",
	/* 0xE9 */	"",
	/* 0xEA */	"",
	/* 0xEB */	"",
	/* 0xEC */	"",
	/* 0xED */	"BeOS",
	/* 0xEE */	"",
	/* 0xEF */	"",

	/* 0xF0 */	"",
	/* 0xF1 */	"",
	/* 0xF2 */	"",
	/* 0xF3 */	"",
	/* 0xF4 */	"",
	/* 0xF5 */	"",
	/* 0xF6 */	"",
	/* 0xF7 */	"",
	/* 0xF8 */	"",
	/* 0xF9 */	"",
	/* 0xFA */	"",
	/* 0xFB */	"VMware File System",
	/* 0xFC */	"VMware Swap",
	/* 0xFD */	"",
	/* 0xFE */	"NT Disk Admin hidden",
	/* 0xFF */	"",
	};
	return (char *)tbl[type & 0xFF];
}

DevTest::Clock_t	DevTest::Clock(void)
{
#if	defined(WIN32)
	return (Clock_t)::clock();
#endif
#if	defined(LINUX)
	struct timeval	tv;

	gettimeofday(&tv, NULL);
	return (Clock_t)tv.tv_sec * CLOCKS_PER_SEC + (Clock_t)tv.tv_usec * CLOCKS_PER_SEC / I64(1000000);
#endif
}

#if	defined(LINUX)
int	strcmpi(const char *s1, const char *s2)
{
	int			ret, c1, c2;

	for (;;)
	{
		c1 = *s1 == '\0' ? 0 : tolower(*s1);
		c2 = *s2 == '\0' ? 0 : tolower(*s2);
		ret = c1 - c2;
		if ( ret || c1 == 0 || c2 == 0 )
			return ret;
		++s1;
		++s2;
	}
}

INT64	_atoi64(const char *s)
{
	INT64	n = 0;
	int		sign = 1;	

	if ( *s == '-' )
	{
		sign = -1;
		s++;
	}
	while ( *s )
	{
		if ( *s < '0' || *s > '9' )
			break;
		n = n * I64(10) + (INT64)(*s - '0');
		s++;
	}
	return sign < 0 ? -n : n;
}

#endif
